/*---------------------------------------------------------------------------\
| Project:          Backup Pyrotechnic System
|
| File:             bps.c
|
| Description:      Routines to process incoming DTMF tones
|
| Global Routines:  <list>
|
| Local Routines:   <list>
|
|    Date      | Notes
|--------------|------------------------------------------------------------
|  05/28/2009  | Largely copied from LV2 Recovery Node
\---------------------------------------------------------------------------*/

//------------------------------------------------------------------------
//  #includes
//------------------------------------------------------------------------

# include <avr/io.h>
# include "bps.h"

//------------------------------------------------------------------------
//  Global Variables
//------------------------------------------------------------------------

uint8_t	bps_state = BPS_DISARMED;

//------------------------------------------------------------------------
//  Module Variables
//------------------------------------------------------------------------

// None

//------------------------------------------------------------------------
//  Local Functions
//------------------------------------------------------------------------

void wait (void)
{
	asm volatile ("nop"::);
}


/*---------------------------------------------------------------------------\
| Function:     initialize
|
| Purpose:      Initialize ATMega168
|
| Discussion:   -
|
| Expects:      -
|
| Arguments:    -
\---------------------------------------------------------------------------*/

void initialize (void)
{
	bps_state = BPS_DISARMED;

	// PORTB

	PORTB = 0x00; // All pins are low
	DDRB =  0xff;  // all are outputs

	PORTC = 0x00; // All pins are low
	DDRC =  0xf0; // All outputs except 0-3 which are DTMF inputs

	PORTD = 0x00; // All pins are low
	DDRD =  0xfb; // 2 is DTMF data valid
}



/*---------------------------------------------------------------------------\
| Function:     fire_pyro
|
| Purpose:      Turn on pyro switches
|
| Discussion:   Fire pyro 1 or 2, but only if we're called with the right key.
|
| Expects:      -
|
| Arguments:    pyro number to fire (1,2) and the pyro key
\---------------------------------------------------------------------------*/

void fire_pyro (uint8_t number, uint8_t key)
{
	uint32_t i;

	if (bps_state == BPS_ARMED)
	{
		if (key == PYRO_FIRE_KEY)
		{
			if (number == 1)
			{
				PYRO_PORT |= PYRO_ONE_MASK;
                BUZZER_PORT |= BUZZER_MASK;
				for (i=0;i<200000;++i)
				{
					wait();
				}
				PYRO_PORT &= ~PYRO_ONE_MASK;
			}
			else if (number == 2)
			{
				PYRO_PORT |= PYRO_TWO_MASK;
                BUZZER_PORT |= BUZZER_MASK;
				for (i=0;i<200000;++i)
				{
					wait();
				}
				PYRO_PORT &= ~PYRO_TWO_MASK;
			}
		}
	}
}




/*---------------------------------------------------------------------------\
| Function:     process_dtmf_tone
|
| Purpose:      Parse incoming DTMF tones from the 2m receiver
|
| Discussion:   - Tone Sequence 1: #*19: Arm the BPS.
|               - Tone Sequence 2: #*28[1,2]: - blow the igniters (only if armed) 1,2
|               - Tone Sequence 3: #*37: Disarm the BPS.
|               - Tone Sequence 4: #*46: -
|
| Expects:      Your mama.
|
| Arguments:    dtmf tone
\---------------------------------------------------------------------------*/

void process_dtmf_tone (uint8_t received_tone)
{
    static uint8_t  tone_number = 0;
    static uint8_t  tone_sequence = 0;

    // If the user starts over at "#" then we reset the
    // state machine to immediately grab the new sequence.

    if (received_tone == DTMF_TONE_POUND)
    {
        tone_number = 0;
        tone_sequence = 0;
        return;
    }

    // If the same tone repeats multiple times, we don't care.
    // This is because the radio might break multiple times
    // while a single tone is being played.

    switch (tone_number)
    {
        case 0:
            if (received_tone == DTMF_TONE_STAR)
                tone_number++;
            else
            {
                tone_number = 0;
                tone_sequence = 0;
            }
            return;

        case 1:
            switch (received_tone)
            {
                // Handle the case of repeated tones first.

                case DTMF_TONE_STAR:
                    return;

                // Now handle new tones.

                case DTMF_TONE_ONE:
                    tone_sequence = 1;
                    tone_number++;
                    return;
                case DTMF_TONE_TWO:
                    tone_sequence = 2;
                    tone_number++;
                    return;
                case DTMF_TONE_THREE:
                    tone_sequence = 3;
                    tone_number++;
                    return;
            }
            tone_number = 0;
            tone_sequence = 0;
            return;

        case 2:

            switch(tone_sequence)
            {
                case 1:

                    // Handle the repeated tone case first
                    if (received_tone == DTMF_TONE_ONE)
                        return;
                    if (received_tone == DTMF_TONE_NINE)
                        //#*19 = Arm the BPS
                        bps_state = BPS_ARMED;
                    break;

                case 2:
                    // Handle the repeated tone case first
                    if (received_tone == DTMF_TONE_TWO)
                        return;
                    if (received_tone == DTMF_TONE_EIGHT)
                        tone_number++;
                    return;

                case 3:
                    // Handle the repeated tone case first
                    if (received_tone == DTMF_TONE_THREE)
                        return;
                    if (received_tone == DTMF_TONE_SEVEN)
                        //#*37 = Disarm the BPS
                        bps_state = BPS_DISARMED;
                        BUZZER_PORT &= ~BUZZER_MASK;
                        PYRO_PORT &= ~PYRO_ONE_MASK;
                        PYRO_PORT &= ~PYRO_TWO_MASK;
                    break;
            }
            tone_number = 0;
            tone_sequence = 0;
            return;

        case 3:

            switch (received_tone)
            {
                // Handle the repeated tone first

                case DTMF_TONE_EIGHT:
                    return;

                case DTMF_TONE_ONE:
                    fire_pyro (1, PYRO_FIRE_KEY);
                    break;

                case DTMF_TONE_TWO:
                    fire_pyro (2, PYRO_FIRE_KEY);
                    break;
            }
            tone_number = 0;
            tone_sequence = 0;
            return;

        default:
            tone_number = 0;
            tone_sequence = 0;
            return;
    }
}





/*---------------------------------------------------------------------------\
| Function:     main
|
| Purpose:      Check for incoming DTMF tones and blink green LED
|
| Discussion:   -
|
| Expects:      -
|
| Arguments:    None
\---------------------------------------------------------------------------*/


int main (void)
{
	uint8_t	dtmf_tone;
	uint8_t	current_holtek_enable = LOW;
	uint8_t	old_holtek_enable = HIGH;
	uint32_t i = 0;

	// Initialize the chip

	initialize();

	// Perform our simple main loop

	while (1)
	{

		// First, check to see if we have a new nibble to parse

		if ((TONE_VALID_PORT & TONE_VALID_MASK) == TONE_VALID_MASK)
		{
			current_holtek_enable = HIGH;
		}
		else
		{
			current_holtek_enable = LOW;
		}

		// Next, "debounce" it: only rising edges matter

		if (current_holtek_enable == HIGH)
		{
			if (old_holtek_enable == LOW)
			{
				// We have a new active high transition; grab the nibble!
				dtmf_tone = PINC & 0x0F;
				process_dtmf_tone(dtmf_tone);
				old_holtek_enable = HIGH;
			}
		}
		else
		{
			old_holtek_enable = LOW;
		}

		// Blink the state LED

		++i;
		if (bps_state == BPS_DISARMED)
		{
			if (i > 100000)
			{
				GRN_LED_TOGGLE = GRN_LED_MASK;	// toggle green LED
				i = 0;
			}
		}
		else if (bps_state == BPS_ARMED)
		{
			if (i > 50000)
			{
				GRN_LED_TOGGLE = GRN_LED_MASK;	// toggle green LED
				BUZZER_TOGGLE = BUZZER_MASK; 	// toggle buzzer
				i = 0;
			}
		}
		else
		{
			bps_state = BPS_DISARMED;
		}
	}
}


